/*
* ffdemo.c FatFs file system demo                           Version 5.2.0
*
* FatFs Demo. Contains code to demonstrate access to FatFs in an increasingly
* partitioned system and ultimately via a portal.
*
* Copyright (c) 2023-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore based on STMicro example
*
* Portable to any ANSI compliant C compiler.
*
*****************************************************************************/

#ifdef FF_DEMO

#include "smxbase.h"
#include "ff.h"
#include "ff_gen_drv.h"
#include "sd_diskio_dma_rtos.h"

#define FILENAME      "0:\\ffdemo.txt"
#define FILEBUF_SZ    255
#define STATUS_COL    10
#define TOP_LINE      2

#ifdef __cplusplus
extern "C" {
#endif
/* function prototypes */
void ffdemo_init(void);
void ffdemo_exit(void);
void ffdemo_main(u32);
void ffdemo_msg(const char* str, int line);
void ffdemo_error(const char* str, int line);
#ifdef __cplusplus
}
#endif

void BSP_SD_WriteCpltCallbackLSRMain(u32);
void BSP_SD_ReadCpltCallbackLSRMain(u32);

/* string literals */
const char hdr[] = "This is STM32 working with FatFs";

/* variables */
TCB_PTR  ffdemo;
u32      failcnt;
u32      passcnt;
FIL      MyFile;        /* file object */
u32      rtext[1024];   /* input buffer */
u32      wtext[1024];   /* output buffer */
FATFS    SDFatFs;       /* File system object for SD card logical drive */
char     SDPath[4];     /* SD card logical drive path */
u32      ffdrtm;        /* ffdemo read time measurement */
u32      ffdwtm;        /* ffdemo write time measurement */
u32      ffdtms;        /* ffdemo time measurement start */
uint8_t  workBuffer[2*_MAX_SS];
LCB_PTR  BSP_SD_WriteCpltCallbackLSR;
LCB_PTR  BSP_SD_ReadCpltCallbackLSR;

void ffdemo_init(void)
{
   /* FatFs init */
   BSP_SD_Init();  /* init SD controller */
   FATFS_LinkDriver(&SD_Driver, SDPath); /* link the micro SD disk I/O driver */
   /* create trusted LSRs for FatFs */
   BSP_SD_WriteCpltCallbackLSR = smx_LSRCreate(BSP_SD_WriteCpltCallbackLSRMain,
     NULL, 0, SMX_FL_TRUST, "SD_WriteCpltLSR", &BSP_SD_WriteCpltCallbackLSR);
   BSP_SD_ReadCpltCallbackLSR = smx_LSRCreate(BSP_SD_ReadCpltCallbackLSRMain,
     NULL, 0, SMX_FL_TRUST, "SD_ReadCpltLSR", &BSP_SD_ReadCpltCallbackLSR);

   ffdemo = smx_TaskCreate(ffdemo_main, PRI_NORM, 1000, 0, "ffdemo", NULL, NULL);
   smx_TaskStart(ffdemo);
}

void ffdemo_exit(void)
{
   /* Unlink the micro SD disk I/O driver */
   FATFS_UnLinkDriver(SDPath);
}

void ffdemo_main(u32)
{
   u32      bytesread;
   u32      byteswritten; 
   u32      i;
   u32      pattern;       /* write pattern */
   FRESULT  res;           /* result code */

   sb_TMInit(); /* initialize time measurement */

   /* Put header in output buffer and start pattern */
   strcpy((char*)wtext, hdr);
   pattern = 0x50505050;

   /* Register the file system object to the FatFs module */
   if(f_mount(&SDFatFs, (TCHAR const*)SDPath, 0) != FR_OK)
   {
      ffdemo_error("FatFs Initialization Error", TOP_LINE);
      return;  /* autostop */
   }

   while (1)
   {
      /* Fill output buffer */
      for (i = sizeof(hdr)/4; i < 1024; i++)
      {
         wtext[i] = pattern;
         pattern += 0x01010101;
      }

      /* Create and Open a new text file object with write access */
      if(f_open(&MyFile, FILENAME, FA_CREATE_ALWAYS | FA_WRITE) != FR_OK)
      {
         ffdemo_error("FatFs Open for Write Error", TOP_LINE);
      }
      else
      {
         /* Write data to the text file and measure time */
         sb_TMStart(&ffdtms);
         res = f_write(&MyFile, wtext, sizeof(wtext), (UINT *)&byteswritten);
         sb_TMEnd(ffdtms, &ffdwtm);

         if((byteswritten == 0) || (res != FR_OK))
         {
            ffdemo_error("FatFs Write or EOF Error", TOP_LINE);
         }
         else
         {
            /* Close the open text file */
            f_close(&MyFile);
         
            /* Open the text file object with read access */
            if(f_open(&MyFile, FILENAME, FA_READ) != FR_OK)
            {
               ffdemo_error("FatFs Open for Read Error", TOP_LINE);
            }
            else
            {
               /* Read data from the text file and measure time */
               sb_TMStart(&ffdtms);
               res = f_read(&MyFile, rtext, sizeof(rtext), (UINT*)&bytesread);
               sb_TMEnd(ffdtms, &ffdrtm);

               if((bytesread == 0) || (res != FR_OK))
               {
                  ffdemo_error("FatFs Read or EOF Error", TOP_LINE);
               }
               else
               {
                  /* Close the open text file */
                  f_close(&MyFile);

                  /* Compare rtext to wtext */
                  for (i = 0; i < 1024; i++)
                  {
                     if (rtext[i] != wtext[i])
                     {
                        ffdemo_error("FatFs Data Comparison Error", TOP_LINE);
                        res = FR_DISK_ERR;
                     }
                  }
    
                  if (res == FR_OK)
                  {
                     /* Success: no errors */
                     ffdemo_msg("FatFs Demo Done", TOP_LINE);
                     passcnt++;
                  }
               }
            }
         }
      }
      smx_TaskSuspend(ffdemo, 25);
//      __asm("ldr r0, =0x20008100 \n\t");   /* cause MMF */
//      __asm("ldr r0, [r0] \n\t");
   }
}

void ffdemo_msg(const char* str, int line)
{
   //Need to open console portal to use these.
   //sb_ConWriteString(STATUS_COL,line,SB_CLR_LIGHTMAGENTA,SB_CLR_BLACK,!SB_CON_BLINK,"                                 ");
   //sb_ConWriteString(STATUS_COL,line,SB_CLR_LIGHTMAGENTA,SB_CLR_BLACK,!SB_CON_BLINK,str);
}

void ffdemo_error(const char* str, int line)
{
   failcnt++;

   //Need to open console portal to use these.
   //sb_ConWriteString(STATUS_COL,line,SB_CLR_LIGHTRED,SB_CLR_BLACK,!SB_CON_BLINK,"                                 ");
   //sb_ConWriteString(STATUS_COL,line,SB_CLR_LIGHTRED,SB_CLR_BLACK,!SB_CON_BLINK,str);
   //smx_DelayMsec(3000);
}  

/* Replaces stub in FatFs */
DWORD get_fattime(void)
{
   DATETIME t;
   u32 dttm = 0;  /* date/time (hi16/lo16) each according to FAT spec */

   sb_GetLocalTime(&t);
   dttm |= (t.wYear << 25);
   dttm |= (t.wMonth << 21);
   dttm |= (t.wDay << 16);
   dttm |= (t.wHour << 11);
   dttm |= (t.wMinute << 5);
   dttm |= (t.wSecond / 2);

   return dttm;
}
#endif /* FF_DEMO */
