//MDI:XYC: The code of this file is modified from STMicroelectronics sample code
//         main.c. See information below.
/**
  ******************************************************************************
  * @file    BSP/Src/main.c
  * @author  MCD Application Team
  * @version V0.1.0
  * @date    07-January-2015
  * @brief   This example code shows how to use the STM32746G-Discovery BSP Drivers
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2015 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */


#include "smxbase.h"
#include "bsp.h"
#ifdef __cplusplus
extern "C" {
#endif

void ClockConfig(void);

#ifdef __cplusplus
}
#endif

static void CPU_CACHE_ENABLE(void);

/**
  * @brief  System Clock Configuration
  *         The system Clock is configured as follow : 
  *            System Clock source            = PLL (HSE)
  *            SYSCLK(Hz)                     = 200000000
  *            HCLK(Hz)                       = 200000000
  *            AHB Prescaler                  = 1
  *            APB1 Prescaler                 = 4
  *            APB2 Prescaler                 = 2
  *            HSE Frequency(Hz)              = 25000000
  *            PLL_M                          = 25
  *            PLL_N                          = 384/400
  *            PLL_P                          = 2
  *            PLL_Q                          = 8
  *            VDD(V)                         = 3.3
  *            Main regulator output voltage  = Scale1 mode
  *            Flash Latency(WS)              = 7
  * @param  None
  * @retval None
  */

void ClockConfig(void)
{
    SystemInit();
    SystemCoreClockUpdate();  //MDI:DM: 08-19-17: Added to init SystemCoreClock since globals haven't been init yet by compiler startup. Fixed by Malcolm Nixon.
    CPU_CACHE_ENABLE();
    /* STM32F7xx HAL library initialization:
       - Configure the Flash prefetch
       - Systick timer is configured by default as source of time base, but user 
         can eventually implement his proper time base source (a general purpose 
         timer for example or other time source), keeping in mind that Time base 
         duration should be kept 1ms since PPP_TIMEOUT_VALUEs are defined and 
         handled in milliseconds basis.
       - Set NVIC Group Priority to 4
       - Low Level Initialization
     */
    HAL_Init();

    RCC_ClkInitTypeDef RCC_ClkInitStruct;
    RCC_OscInitTypeDef RCC_OscInitStruct;
    HAL_StatusTypeDef ret = HAL_OK;

    /* Enable Power Control clock */
    __HAL_RCC_PWR_CLK_ENABLE();

    /* The voltage scaling allows optimizing the power consumption when the device is
     clocked below the maximum system frequency, to update the voltage scaling value 
     regarding system frequency refer to product datasheet.  */
    __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);


    /* Enable HSE Oscillator and activate PLL with HSE as source */
    RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
    RCC_OscInitStruct.HSEState = RCC_HSE_ON;
    RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
    RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
    RCC_OscInitStruct.PLL.PLLM = 25;
#if SB_USB_FS  //MDI:XYC: 03-17-16: Added this case.
    RCC_OscInitStruct.PLL.PLLN = 384;
#else 
    RCC_OscInitStruct.PLL.PLLN = 400;
#endif
    RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
    RCC_OscInitStruct.PLL.PLLQ = 8;
    ret = HAL_RCC_OscConfig(&RCC_OscInitStruct);
    if(ret != HAL_OK)
    {
        while(1) { ; }
    }
  
    /* Activate the OverDrive to reach the 200 MHz Frequency */  
    ret = HAL_PWREx_ActivateOverDrive();
    if(ret != HAL_OK)
    {
        while(1) { ; }
    }
  
    /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 clocks dividers */
    RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
    RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
    RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
    RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
    RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

    ret = HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_7);  //MDI:DM: 08-07-19: Changed 5 to 7 to avoid flakiness seen on STM32F769I-EVAL. Consistent with most STMicro HAL examples.
    if(ret != HAL_OK)
    {
        while(1) { ; }
    }  
}

static void CPU_CACHE_ENABLE(void)
{
    __DSB();
    
    /* Disable the MPU */
    MPU->CTRL = 0;

    /* Configure Region 0 for internal SRAM as Normal memory type with WBWA and shareable (for coherency with DMA) */
    MPU->RBAR = 0x20000000 |                    /* Region address */
                MPU_RBAR_VALID_Msk |            /* Region configured */
                (0 << MPU_RBAR_REGION_Pos);     /* Region 0 */
    MPU->RASR = (0x0 << MPU_RASR_XN_Pos) |      /* Allow instruction fetches (may contain code) */
                (0x3 << MPU_RASR_AP_Pos) |      /* R/W */
                (0x1 << MPU_RASR_TEX_Pos) |     /* WBWA */
                (0x0 << MPU_RASR_S_Pos) |       /* Not Shareable */
                (0x1 << MPU_RASR_C_Pos) |       /* Cacheable */
                (0x1 << MPU_RASR_B_Pos) |       /* Bufferable */
                (0x00 << MPU_RASR_SRD_Pos ) |   /* All sub-regions configured */
                (0x11 << MPU_RASR_SIZE_Pos) |   /* 512KB */
                MPU_RASR_ENABLE_Msk;            /* Enable */

    /* Configure Region 1 for SDRAM as Normal memory type with WBWA and shareable (for coherency with DMA) */
    MPU->RBAR = 0xC0000000 |                    /* Region address */
                MPU_RBAR_VALID_Msk |            /* Region configured */
                (1 << MPU_RBAR_REGION_Pos);     /* Region 1 */
    MPU->RASR = (0x0 << MPU_RASR_XN_Pos) |      /* Allow instruction fetches (may contain code) */
                (0x3 << MPU_RASR_AP_Pos) |      /* R/W */
                (0x1 << MPU_RASR_TEX_Pos) |     /* WBWA */
                (0x0 << MPU_RASR_S_Pos) |       /* Not shareable */
                (0x1 << MPU_RASR_C_Pos) |       /* Cacheable */
                (0x1 << MPU_RASR_B_Pos) |       /* Bufferable */
                (0x00 << MPU_RASR_SRD_Pos ) |   /* All sub-regions configured */
                (0x18 << MPU_RASR_SIZE_Pos) |   /* 32MB */
                MPU_RASR_ENABLE_Msk;            /* Enable */

    /* Enable the MPU, and use the default map for any unspecified region */
    MPU->CTRL = MPU_CTRL_PRIVDEFENA_Msk | MPU_CTRL_ENABLE_Msk;
    __DSB();
    __ISB();

    /* Invalidate I&D Cache */
    SCB_InvalidateICache();
    SCB_InvalidateDCache();
    
    /* Enable branch prediction */
    SCB->CCR |= (1 << 18); 
    __DSB();

    /* Enable I&D Cache */
    SCB_EnableICache();
    //SCB_EnableDCache();
}
/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
