/*
* startup.c                                                 Version 5.2.0
*
* Startup code extension. Called early in the compiler's startup code.
* You can add any other early init code here, as desired.
*
* IAR:        Called by __iar_program_start() in cmain.s
* CrossWorks: Called by _start in thumb_crt0.s
*
* Copyright (c) 2009-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore, Xingsheng Wan, XiaoYi Chen
*
*****************************************************************************/

#include "smxbase.h"
#include "bsp.h"

/* Macro for checking if the code is located in ROM */
#define SB_CODE_IN_ROM(x)  ((0x00200000 <= x) && (x < 0x002FFFFF)) || \
                           ((0x08000000 <= x) && (x < 0x080FFFFF))

#ifdef __cplusplus
extern "C" {
#endif

extern const intvec_elem __vector_table[];
extern void PinConfig(void);  /* pin.c */
static volatile u32 uwTick;

int __low_level_init(void)
{
   u32 i;
#if defined(__IAR_SYSTEMS_ICC__) && defined(SB_ICF_NEW)
   u32 *evt_ram = (u32*)__section_begin("EVT");
#else
   u32 *evt_ram = (u32 *)SB_EVT_RAM_BASE;
#endif
   u32 *evt_rom = (u32 *)__vector_table;

   /* Set Vector Table Offset Register to new EVT in ROM temporarily. */
   *ARMM_NVIC_VTOR = (u32)evt_rom;

#if __ARMVFPV3__ || __ARMVFPV4__ || __ARMVFPV5__
   /* Enable FPU. Compiler startup should have already done next statement. */
   *ARMM_SCB_CPACR |= ((3 << (10 * 2)) | (3 << (11 * 2)));
   asm("dsb");
   asm("isb");
// *ARMM_FPU_FPCCR &= 0x3FFFFFFF;  /* Disable FPU auto save and lazy stacking */
   *ARMM_FPU_FPCCR &= 0xBFFFFFFF;  /* Disable FPU lazy stacking <1> */
#endif

   /* Init these before HAL code runs since .bss and .data not init yet. */
   sb_tick_init_done = FALSE;
   uwTick = 0;

   /* Only initialize clock and ex-RAMs when code located in ROM. */ 
   if(SB_CODE_IN_ROM((u32)__low_level_init))
   {
      sb_ClocksInit();

      BSP_SDRAM_Init();        
#if defined(SB_BRD_STMICRO_STM32746GEVAL) || defined(SB_BRD_STMICRO_STM32F769IEVAL) || defined(SB_BRD_STMICRO_STM32F779IEVAL)
      BSP_SRAM_Init();        
#endif        
   } 
    
   sb_IntCtrlInit();

   /* Copy EVT from Flash to SRAM because SMX hooks vectors dynamically. */
   if (evt_ram != evt_rom)
   {
      for (i = 0; i < (16+SB_IRQ_MAX+1); i++)
         evt_ram[i] = evt_rom[i];
   }
   /* Set Vector Table Offset Register to new EVT location. */
   *ARMM_NVIC_VTOR = (u32)evt_ram;

   PinConfig();

   sb_SSFillMost();  /* fill system stack */

   /* USER: You can add any other early init code here. sb_PeripheralsInit()
            is another place that runs later. Decide which is appropriate.
   */

   return 1;  /* 1 tells IAR startup code to run seg_init; 0 skips */
}

#if SMX_CFG_MPU
/*+++++++++++++++++++++++ LIMITED SWI/SVC API (UMODE) ++++++++++++++++++++++*/
#include "xapiu.h"  /* need sbu_ and smxu_ prototypes */
#include "xapip.h"  /* but don't want mapping macros */
#endif

/*
   The following 3 functions are declared and have weak definitions in
   STMicro Library v1.5. They use the tick exception to generate delay
   for peripheral initialization. We have to override them here because
   SMX takes over the tick exception.
*/
void HAL_IncTick(void)
{
   uwTick++;
}

u32 HAL_GetTick(void)
{  
  #if SMX_CFG_MPU
   if (sb_IN_UMODE())
   {
      return smxu_SysEtimeGet(SMX_FL_MSEC);  /*<3>*/
   }
   else
  #endif
   {
      if(sb_tick_init_done)  /*<2>*/
         return smx_SysEtimeGet(SMX_FL_MSEC);
      else
         return uwTick;
   }
}

void HAL_Delay(u32 Delay)
{
  #if SMX_CFG_MPU
   if(sb_IN_UMODE() || sb_tick_init_done)  /*<2> <3>*/
  #else
   if(sb_tick_init_done)  /*<2>*/
  #endif
   {
      sb_DelayMsec(Delay);
   }
   else
   {
      u32 tickstart;
      tickstart = HAL_GetTick();
      while((HAL_GetTick() - tickstart) < Delay){}
   }
}

#ifdef __cplusplus
}
#endif

/* Notes:
   1. Not reliable for tasks.
   2. Necessary to check sb_tick_init_done because HAL_GetTick() and
      HAL_Delay() are called during low-level startup code, before smx
      tick is initialized in main(), so we use HAL uwTick until then.
      Also, it can be read directly because this case runs in pmode.
   3. Unnecessary to check sb_tick_init_done in umode case because
      tick init was done in pmode before we ever switch to umode.
*/