/*
* barmm.h                                                   Version 5.2.0
*
* ARM-M (e.g. Cortex-M) definitions and macros.
*
* Copyright (c) 2008-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
*****************************************************************************/

#ifndef SB_BARMM_H
#define SB_BARMM_H

/* Configuration */  /* USER: You may wish to change these settings. */

#define SB_ARMM_DISABLE_WITH_BASEPRI       0  /* Change barmm.inc too. See Note 1 below. */
#define SB_ARMM_BASEPRI_VALUE           0x20  /* Change barmm.inc too. See Note 2 below. */

#define SB_ARMM_DISABLE_TRAP               0  /* If 1 use alternate DISABLE/ENABLE that do DEBUGTRAP (see below). */

#if defined(SMX_FRPORT)
/* FreeRTOS port: See note <5> below and then comment out this #error. */
#error Set SB_ARMM_DISABLE_WITH_BASEPRI to 1 and SB_ARMM_BASEPRI_VALUE in barmm.h and barmm_iar.inc.
#endif

/* Common Register Definitions (common to all ARM-M processors) */
/* Here we define just the registers smx needs (and a few it might need). */
/* Defining as pointers allows adding to them before dereferencing. */

#define ARMM_FPU_FPCCR               ((vu32 *)0xE000EF34)  /* Floating-point Context Control Register */
#define ARMM_MPU_TYPE                ((vu32 *)0xE000ED90)  /* MPU Type */
#define ARMM_MPU_CTRL                ((vu32 *)0xE000ED94)  /* MPU Control */
#define ARMM_MPU_RNR                 ((vu32 *)0xE000ED98)  /* MPU Region Number */
#define ARMM_MPU_RBAR                ((vu32 *)0xE000ED9C)  /* MPU Region Base Address */
#if SB_CPU_ARMM7
#define ARMM_MPU_RASR                ((vu32 *)0xE000EDA0)  /* MPU Region Base Attribute and Size */
#elif SB_CPU_ARMM8
#define ARMM_MPU_RLAR                ((vu32 *)0xE000EDA0)  /* MPU Region Limit Address */
#define ARMM_MPU_MAIR0               ((vu32 *)0xE000EDC0)  /* MPU MAIR0 */
#define ARMM_MPU_MAIR1               ((vu32 *)0xE000EDC4)  /* MPU MAIR1 */
#endif
#define ARMM_NVIC_SYSTICK_CTRL       ((vu32 *)0xE000E010)  /* SysTick Control and Status */
#define ARMM_NVIC_SYSTICK_RELOAD     ((vu32 *)0xE000E014)  /* SysTick Reload */
#define ARMM_NVIC_SYSTICK_CURRENT    ((vu32 *)0xE000E018)  /* SysTick Current Value */
#define ARMM_NVIC_SYSTICK_CAL        ((vu32 *)0xE000E01C)  /* SysTick Calibration Value */
#define ARMM_NVIC_IRQ_SET_EN_BASE    ((vu32 *)0xE000E100)  /* IRQ Set Enable First Reg */
#define ARMM_NVIC_IRQ_CLR_EN_BASE    ((vu32 *)0xE000E180)  /* IRQ Clear Enable First Reg */
#define ARMM_NVIC_IRQ_SET_PEND_BASE  ((vu32 *)0xE000E200)  /* IRQ Set Enable First Reg */
#define ARMM_NVIC_IRQ_CLR_PEND_BASE  ((vu32 *)0xE000E280)  /* IRQ Clear Enable First Reg */
#define ARMM_NVIC_IRQ_PRI_BASE       ((vu32 *)0xE000E400)  /* IRQ Priority First Reg */
#define ARMM_NVIC_INT_CTRL           ((vu32 *)0xE000ED04)  /* Interrupt Control State Register */
#define ARMM_NVIC_VTOR               ((vu32 *)0xE000ED08)  /* Vector Table Offset Register */
#define ARMM_NVIC_APPINT_RST_CTRL    ((vu32 *)0xE000ED0C)  /* App Interrupt and Reset Control Register */
#define ARMM_NVIC_CONFIG_CTRL        ((vu32 *)0xE000ED14)  /* Configuration Control */
#define ARMM_NVIC_SYSEXC_PRI_0704    ((vu32 *)0xE000ED18)  /* Exceptions  4- 7 Priority */
#define ARMM_NVIC_SYSEXC_PRI_1108    ((vu32 *)0xE000ED1C)  /* Exceptions  8-11 Priority */
#define ARMM_NVIC_SYSEXC_PRI_1512    ((vu32 *)0xE000ED20)  /* Exceptions 12-15 Priority */
#define ARMM_NVIC_SYSHAN_CTRL_STATE  ((vu32 *)0xE000ED24)  /* System Handler Control and State */
#define ARMM_SCB_CPACR               ((vu32 *)0xE000ED88)  /* Coprocessor Access Control Register */
#define ARMM_UFSR                    ((vu16 *)0xE000ED2A)  /* Usage Fault Status Register (upper half of CFSR) */

/* Common ARM-M Register Flags */

#define ARMM_FL_PENDSVSET            0x10000000
#define ARMM_FL_RETTOBASE            0x00000800
#define ARMM_FL_STKOF                0x0010                /* Stack Overflow in UFSR */

#define SB_CACHE_LINE      16 /* minimum w or w/o cache */
#define SB_DATA_ALIGN      4  /* minimum alignment for 32-bit data writes */
                              /* 1 is supported except areas that are Device Memory Type <3> */
#define SB_STACK_ALIGN     8  /* minimum alignment of stack elements */
                              /* 8 is the minimum specified by AAPCS */

#define sb_HALTEXEC()      while(1) {}
#define sb_SSP_INIT()      sb_ssp = (u8 *)sb_SS_GET_TOP() + sb_SS_GET_SIZE();

/*
   CPU_FL is defined to be the size of the flags register on
   the processor. This is used by sb_IntStateSaveDisable() and
   sb_IntStateRestore() to save the state of the interrupt flag(s).
*/
typedef u32 CPU_FL;

/* IRQ_PERM is a range of IRQs permitted to be masked when using the MPU */
typedef struct {
   u8 irqmin;
   u8 irqmax;
} IRQ_PERM;


#if defined(__CC_ARM)
/*===========================================================================*
*                                  ARM C/C++                                 *
*===========================================================================*/

#error Implement ARM C/C++ case here in barmm.h.


#elif defined(__GNUC__)
/*===========================================================================*
*                                  GNU C/C++                                 *
*===========================================================================*/

#if defined(__ARMEL__)
#define __LITTLE_ENDIAN__ 1
#elif defined(__ARMEB__)
#define __LITTLE_ENDIAN__ 0
#else
#error Define __LITTLE_ENDIAN__ in __GNUC__ case in barmm.h.
#endif

#define sb_BREAKPOINT()  __asm("bkpt 0 \n\t");  /* for use in sb_DEBUGTRAP() */

#if SB_ARMM_DISABLE_WITH_BASEPRI
/* GNU doesn't seem to have intrinsics to set BASEPRI, etc. like IAR.
   Here we need to save r0 or make a function call to preserve it.
*/
#define sb_INT_DISABLE() \
   { \
      __asm("push {r0} \n\t"); \
      __asm("mov  r0, #" stringify_expanded(SB_ARMM_BASEPRI_VALUE) "\n\t"); \
      __asm("msr  basepri, r0 \n\t");         /* BASEPRI = SB_ARMM_BASEPRI_VALUE */ \
      __asm("pop  {r0} \n\t"); \
   }

#define sb_INT_ENABLE() \
   { \
      __asm("push {r0} \n\t"); \
      __asm("mov  r0, #0 \n\t"); \
      __asm("msr  basepri, r0 \n\t");         /* BASEPRI = 0 */ \
      __asm("pop  {r0} \n\t"); \
   }
#else
#define sb_INT_DISABLE()     __asm("cpsid i \n\t");  /* PRIMASK = 1 */
#define sb_INT_ENABLE()      __asm("cpsie i \n\t");  /* PRIMASK = 0 */
#endif

/* Special for ARM-M */
#define sb_INT_DISABLEF()    __asm("cpsid f \n\t");  /* FAULTMASK = 1 */
#define sb_INT_ENABLEF()     __asm("cpsie f \n\t");  /* FAULTMASK = 0 */
#define sb_SVC(id)           __asm("svc %0 \n\t" : : "i" (id));

/* System Stack / Startup Stack macros used by smxBase and smx */
#if defined(__CROSSWORKS_ARM)
extern u8 __stack_start__[]; /* defined in linker command file */
extern u8 __stack_end__[];   /* defined in linker command file */
#define sb_SS_GET_TOP()  (&__stack_start__)
#define sb_SS_GET_SIZE() ((u32)&__stack_end__ - (u32)&__stack_start__)
#else
#error Implement sb_SS_ macros for your GNU.
#endif

#elif defined(__ICCARM__)
/*===========================================================================*
*                               IAR ARM C/C++                                *
*===========================================================================*/

/* __LITTLE_ENDIAN__ is automatically defined by IAR. */

#define sb_BREAKPOINT()      __asm volatile ("bkpt 0");  /* for use in sb_DEBUGTRAP() */

#if !SB_ARMM_DISABLE_TRAP
/* Normal versions */
 #if SB_ARMM_DISABLE_WITH_BASEPRI
#define sb_INT_DISABLE()     __set_BASEPRI(SB_ARMM_BASEPRI_VALUE);  /* IAR intrinsic */
#define sb_INT_ENABLE()      __set_BASEPRI(0);
 #else
#define sb_INT_DISABLE()     __asm volatile ("cpsid i");  /* PRIMASK = 1 */
#define sb_INT_ENABLE()      __asm volatile ("cpsie i");  /* PRIMASK = 0 */
 #endif

#else
/* MPU debug versions that trap on use in unprivileged mode. These silently nop
   in unprivileged mode. Need to find them to change code to do something else.
   Rebuild all code if enabled/disabled.
*/
 #if SB_ARMM_DISABLE_WITH_BASEPRI
#define sb_INT_DISABLE()     if (sb_IN_UMODE()) \
                                sb_DEBUGTRAP() \
                             else \
                                __set_BASEPRI(SB_ARMM_BASEPRI_VALUE);  /* IAR intrinsic */
#define sb_INT_ENABLE()      if (sb_IN_UMODE()) \
                                sb_DEBUGTRAP() \
                             else \
                                __set_BASEPRI(0);
 #else
#define sb_INT_DISABLE()     if (sb_IN_UMODE()) \
                                sb_DEBUGTRAP() \
                             else \
                                __asm volatile ("cpsid i");  /* PRIMASK = 1 */
#define sb_INT_ENABLE()      if (sb_IN_UMODE()) \
                                sb_DEBUGTRAP() \
                             else \
                                __asm volatile ("cpsie i");  /* PRIMASK = 0 */
 #endif
#endif

/* Special for ARM-M */
#define sb_INT_DISABLEF()    __asm volatile ("cpsid f");  /* FAULTMASK = 1 */
#define sb_INT_ENABLEF()     __asm volatile ("cpsie f");  /* FAULTMASK = 0 */
#define sb_IN_SVC()          ((*ARMM_NVIC_INT_CTRL & 0xFF) == 0x0B) /* in SVC handler? */
#define sb_IN_UMODE()        (__get_MSP() == 0)           /* <4> */
#define sb_SVC(id)           __asm("svc %0" : : "i" (id));

/* System Stack / Startup Stack macros used by smxBase and smx */
#pragma section = "CSTACK" /* defined in linker command file (.icf) */
#define sb_SS_GET_TOP()      __section_begin("CSTACK")
#define sb_SS_GET_SIZE()     __section_size("CSTACK")

#else
/*===========================================================================*
*                                  -----                                     *
*===========================================================================*/
#error Define the porting macros for your compiler here in barmm.h.

#endif  /* compiler checks */
/*==========================================================================*/


#if defined(SMX_BT_DEBUG)
#define sb_DEBUGTRAP()   { sb_BREAKPOINT(); }
#else
#define sb_DEBUGTRAP()   { }
#endif

#if !defined(__LITTLE_ENDIAN__)
#error Define __LITTLE_ENDIAN__ 0/1 in your compiler section of barmm.h.
#endif
#define SB_CPU_BIG_ENDIAN    !(__LITTLE_ENDIAN__)
#define SB_CPU_LITTLE_ENDIAN (__LITTLE_ENDIAN__)


/*
Notes:

1. SB_ARMM_DISABLE_WITH_BASEPRI vs. PRIMASK: Disabling interrupts with BASEPRI
   leaves the top one or more priority levels unmasked vs. PRIMASK which
   disables all interrupts (except faults). PRIMASK is simpler and safer,
   but using BASEPRI gives one or more non-maskable priority levels, which
   may be necessary for motor control ISRs and similar. If you use BASEPRI,
   DO NOT define smx ISRs (those that use smx_ISR_ENTER/EXIT and invoke LSRs)
   at the unmaskable priority level(s).

2. SB_ARMM_BASEPRI_VALUE: When using BASEPRI to disable interrupts, those
   with priority equal or lower (higher number) than this setting are
   inhibited. Those higher (lower number) are non-maskable. Remember that
   the number of priority bits can differ on different ARM-M processors.
   For example, if the processor uses a 3-bit priority, priorities are
   0x00, 0x20, 0x40, etc. Setting this threshold to 0x20 means that 0x00
   is non-maskable. If the processor uses a 4-bit priority, priorities are
   0x00, 0x10, 0x20, etc. Setting this threshold to 0x20 means that 0x00
   and 0x10 are non-maskable. You may change this setting as desired. But
   heed the warning in Note 1 and change any ISR priorities if necessary.

3. ARM-M generally supports 1-byte alignment for data reads/writes, but for
   areas designated Device Memory Type, alignment must be 4-byte aligned to
   avoid faults. To avoid problems, we set this to 4. You can set to 1 if
   this limitation doesn't apply to your target or if this alignment value
   is used for code that does not access areas of Device Memory Type.

4. sb_IN_UMODE() uses a trick, since CONTROL.nPriv cannot be checked in umode.
   Accessing CONTROL, MSP, PSP, etc. returns 0 in umode. Since MSP is always
   non-zero, we must be in umode if it reads as 0.

5. FreeRTOS port (FRPort): Set SB_ARMM_DISABLE_WITH_BASEPRI to 1 and
   SB_ARMM_BASEPRI_VALUE the same as configMAX_SYSCALL_INTERRUPT_PRIORITY
   here and in barmm_iar.inc. It is simplest to set ours to the same numeric
   value rather than to use their constant, to avoid include file problems
   and an undefined symbol for assembly. Also it is recommended to change
   their value to match our default, or adjust priorities in our irqtable.c.
   irqtable.c was set assuming BASEPRI is 0x20, but in the FreeRTOS release
   we worked with, it was set to 0x40.
*/

#endif /* SB_BARMM_H */

