/*
* isrshells.c                                               Version 5.2.0
*
* ISR Shell and Vector Hooking Functions
*
* The shells and functions in this file can handle shared interrupts.
* A different issue is the need for assembly shells to achieve the
* proper ISR prolog needed by smx for some processor architectures.
* Those shells are in a separate file which is linked only for processors
* that require it. This file should be linked in all cases.
*
* Copyright (c) 2010-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: Yingbo Hu, Richard Ames, and David Moore
*
*****************************************************************************/

#include "smxbase.h"

#if defined(SB_CPU_ARM)
#include "armdefs.h"
#endif

/* Interrupt Shell Type */
#define SB_ISHL_AISR 0  /* assembly ISR (fully written in assembly) */
#define SB_ISHL_CFUN 1  /* C function (called by dispatcher which does ISR_ENTER/EXIT) */
#define SB_ISHL_CISR 2  /* C ISR (fully written in C) */

#if defined(SB_CPU_ARMM)
#define SB_ISR_SHELL_TYPE SB_ISHL_CISR
#elif defined(SB_CPU_ARM) && IRQ_DISPATCH_IN_HDW
#define SB_ISR_SHELL_TYPE SB_ISHL_AISR
#elif defined(SB_CPU_ARM) && !IRQ_DISPATCH_IN_HDW
#define SB_ISR_SHELL_TYPE SB_ISHL_CFUN
#else
#define SB_ISR_SHELL_TYPE SB_ISHL_CISR
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* sb_ISRInfo[] cell structure */
typedef struct
{
    int      irq;
    uint     par;
    ISRC_PTR fun;
    ISR_PTR  funsv;
} sb_ISR_INFO;

static sb_ISR_INFO sb_ISRInfo[SB_CFG_IRQ_MAX_NUM];
static int sb_ISRInited = 0;

#if SMX_CFG_EVB || SMX_EVB
static void* sb_ISRHandleArray[SB_CFG_IRQ_MAX_NUM];
#else
#define smx_EVB_LOG_ISR(isr)
#define smx_EVB_LOG_ISR_RET(isr)
#endif

void sb_ISR(uint index);

#if (SB_ISR_SHELL_TYPE == SB_ISHL_AISR)
/* ISR shells for assembly ISRs, defined in assembly file */
void sb_ISR0(void);
void sb_ISR1(void);
void sb_ISR2(void);
void sb_ISR3(void);
void sb_ISR4(void);
void sb_ISR5(void);
void sb_ISR6(void);
void sb_ISR7(void);
void sb_ISR8(void);
void sb_ISR9(void);

#elif (SB_ISR_SHELL_TYPE == SB_ISHL_CFUN)
/* ISR shells in C, called by dispatcher which does ISR_ENTER/EXIT */
void sb_ISR0(void)
{
    sb_ISR(0);
}
#if SB_CFG_IRQ_MAX_NUM > 1
void sb_ISR1(void)
{
    sb_ISR(1);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 2
void sb_ISR2(void)
{
    sb_ISR(2);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 3
void sb_ISR3(void)
{
    sb_ISR(3);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 4
void sb_ISR4(void)
{
    sb_ISR(4);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 5
void sb_ISR5(void)
{
    sb_ISR(5);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 6
void sb_ISR6(void)
{
    sb_ISR(6);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 7
void sb_ISR7(void)
{
    sb_ISR(7);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 8
void sb_ISR8(void)
{
    sb_ISR(8);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 9
void sb_ISR9(void)
{
    sb_ISR(9);
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 10
#error Add more ISR shells to CFUN case in isrshells.c to match SB_CFG_IRQ_MAX_NUM.
#endif

#elif (SB_ISR_SHELL_TYPE == SB_ISHL_CISR)
/* ISR shells in C, which do ISR_ENTER/EXIT */

#if defined(SB_CPU_ARMM) && SMX_CFG_MPU
/*
   For smx on ARM-M with MPU enabled:
   - These are put into the .sys_text section, which is put into the
     sys_code region in MPU slot 7 if the MPU is on, and background
     region is normally off. Thus, they are accessible whenever an
     IRQ occurs.
   - ISR_ENTER() enables the background region first to permit the ISR
     to access anything it needs. ISR_EXIT() disables it.
*/
#endif
__interdecl void __interrupt sb_ISR0(void)
{
    smx_ISR_ENTER();
    sb_ISR(0);
    smx_ISR_EXIT();
}
#if SB_CFG_IRQ_MAX_NUM > 1
__interdecl void __interrupt sb_ISR1(void)
{
    smx_ISR_ENTER();
    sb_ISR(1);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 2
__interdecl void __interrupt sb_ISR2(void)
{
    smx_ISR_ENTER();
    sb_ISR(2);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 3
__interdecl void __interrupt sb_ISR3(void)
{
    smx_ISR_ENTER();
    sb_ISR(3);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 4
__interdecl void __interrupt sb_ISR4(void)
{
    smx_ISR_ENTER();
    sb_ISR(4);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 5
__interdecl void __interrupt sb_ISR5(void)
{
    smx_ISR_ENTER();
    sb_ISR(5);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 6
__interdecl void __interrupt sb_ISR6(void)
{
    smx_ISR_ENTER();
    sb_ISR(6);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 7
__interdecl void __interrupt sb_ISR7(void)
{
    smx_ISR_ENTER();
    sb_ISR(7);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 8
__interdecl void __interrupt sb_ISR8(void)
{
    smx_ISR_ENTER();
    sb_ISR(8);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 9
__interdecl void __interrupt sb_ISR9(void)
{
    smx_ISR_ENTER();
    sb_ISR(9);
    smx_ISR_EXIT();
}
#endif
#if SB_CFG_IRQ_MAX_NUM > 10
#error Add more ISR shells to CISR case in isrshells.c to match SB_CFG_IRQ_MAX_NUM.
#endif
#endif /* SB_ISR_SHELL_TYPE */

/* Array of IRS shells defined above. */
const static ISR_PTR sb_ISRArray[] =
{
    (ISR_PTR)sb_ISR0,
#if SB_CFG_IRQ_MAX_NUM > 1
    (ISR_PTR)sb_ISR1,
#endif
#if SB_CFG_IRQ_MAX_NUM > 2
    (ISR_PTR)sb_ISR2,
#endif
#if SB_CFG_IRQ_MAX_NUM > 3
    (ISR_PTR)sb_ISR3,
#endif
#if SB_CFG_IRQ_MAX_NUM > 4
    (ISR_PTR)sb_ISR4,
#endif
#if SB_CFG_IRQ_MAX_NUM > 5
    (ISR_PTR)sb_ISR5,
#endif
#if SB_CFG_IRQ_MAX_NUM > 6
    (ISR_PTR)sb_ISR6,
#endif
#if SB_CFG_IRQ_MAX_NUM > 7
    (ISR_PTR)sb_ISR7,
#endif
#if SB_CFG_IRQ_MAX_NUM > 8
    (ISR_PTR)sb_ISR8,
#endif
#if SB_CFG_IRQ_MAX_NUM > 9
    (ISR_PTR)sb_ISR9,
#endif
#if SB_CFG_IRQ_MAX_NUM > 10
#error Add more elements to sb_ISRArray[] in isrshells.c to match SB_CFG_IRQ_MAX_NUM.
#endif
};


/* sb_ISR() is the common routine called from each ISR shell. It calls the 
   ISR for an IRQ level in sb_ISRInfo[] and logs it in EVB.
   Note: Non-hooked IRQs -> a default handler in many BSPs.
*/
void sb_ISR(uint index)
{
    smx_EVB_LOG_ISR(sb_ISRHandleArray[index]);
    sb_ISR_INFO isrinfo = sb_ISRInfo[index];
   #if !defined(SB_CPU_ARMM)
    sb_IRQClear(isrinfo.irq);
   #endif
    isrinfo.fun(isrinfo.par);
    sb_IRQEnd(isrinfo.irq);
    smx_EVB_LOG_ISR_RET(sb_ISRHandleArray[index]);
}

/* sb_ISRInit() loads inactive state into all cells of the sb_ISRInfo[].
*/
static void sb_ISRInit(void)
{
    int i;
    for(i = 0; i < SB_CFG_IRQ_MAX_NUM; i++)
    {
        sb_ISRInfo[i].irq = -1;
        sb_ISRInfo[i].fun = NULL;
        sb_ISRInfo[i].funsv = NULL;
    }
}

/* sb_ISRInstall() performs hooking of IRQ levels with ISR shells:
   Places an ISR address with parameter into sb_ISRInfo[] array. On first call
   (sb_ISRInited == 0) all cells of the array are put into an inactive state.
   On each subsequent call, an inactive cell is found to load an ISR address
   and ISR parameter. If event buffer is enabled, a name is also loaded into
   the ISR handle array.
*/
BOOLEAN sb_ISRInstall(int irq_num, uint par, ISRC_PTR fun, const char* name)
{
    int i;
    if(!sb_ISRInited) /* first call */
    {
        sb_ISRInited = 1;
        sb_ISRInit();
    }
    for(i = 0; i < SB_CFG_IRQ_MAX_NUM; i++) /* search for first inactive level */
    {
        if(sb_ISRInfo[i].irq == -1)
        {
            sb_ISRInfo[i].irq = irq_num;
            sb_ISRInfo[i].par = par;
            sb_ISRInfo[i].fun = fun;

            /* save old ISR address in vector and load address of IRQ shell into vector */
            sb_ISRInfo[i].funsv = sb_IRQVectGet(irq_num, NULL);
            sb_IRQConfig(irq_num); 
            sb_IRQVectSet(irq_num, sb_ISRArray[i]);

           #if SMX_CFG_EVB || SMX_EVB
            /* load name into ISR handle array */
            sb_ISRHandleArray[i] = smx_SysPseudoHandleCreate(); 
            smx_HT_ADD(sb_ISRHandleArray[i], name);
           #endif
            return TRUE;
        }
    }
    return FALSE;
}

/* sb_ISRRestore() hooks the old ISR to the IRQ level.
*/
BOOLEAN sb_ISRRestore(int irq_num)
{
    int i;
    for(i = 0; i < SB_CFG_IRQ_MAX_NUM; i++) /* search for irq level */
    {
        if(sb_ISRInfo[i].irq == irq_num)
        {
            sb_ISRInfo[i].irq = -1;
            sb_ISRInfo[i].fun = NULL;

            /* put back old ISR address */
            sb_IRQConfig(irq_num); 
            sb_IRQVectSet(irq_num, sb_ISRInfo[i].funsv);
            return TRUE;
        }
    }
    return FALSE;
}

#ifdef __cplusplus
}
#endif

