/*
* smxaware.h                                                Version 5.2.0
*
* smxAware and smxAware Live command defines and structures.
* This file is shared with the smxAware Live server (target) and client (PC).
*
* Copyright (c) 1999-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Marty Cochran
*
*****************************************************************************/

#ifndef SMX_SMXAWARE_H
#define SMX_SMXAWARE_H

#if defined(SMXAWARE)

#define SMXAWARE_DATA_VERSION     0x400
#define ENABLE_SMXOBJECT_SYMBOLS  1

#define MP_MPU_TYPE_ARMM7   0x01
#define MP_MPU_TYPE_ARMM8   0x02

/* build settings for smx_BuildFlags (32 bit flags max) */
#define SMX_CFG_TOKENS_DEFINED       0x0001

typedef struct
{
   u32  length;                         /* length of ring buffer */
   u32  next;                           /* next byte to write */
   u32  unused;
   u32  version;
   u8   rng_buf[SMX_CFG_SA_PRINT_RING_SIZE+4];  /* +4 explained in Note 1 at the end of this file */
} SA_PRINT;

#if ((SA_PRINT_RING_SIZE/4) * 4 != SA_PRINT_RING_SIZE)
#error SA_PRINT_RING_SIZE must be divisible by 4
#endif

#if defined(SMXAWARE_LIVE)
#define SAL_PORT_NUM              40002         /* target and smxAwareLive must match */
#define SAL_XFER_BUFF_LENGTH      300
#define SAL_NAME_LEN              22
#define SAL_DEFAULT_MESSAGE_SIZE  16; /* set default, sizeof messageLength + command + address + length */
#define SAL_COMMAND_HEADER_SIZE   12  /* sizeof( command + address + length) */
#define SMXAWARE_SYM_NAME_LEN     20


/* commands to the target */
#define SAL_ALIVE_COMMAND            0x01 /* First command to be send. Used to establish Endian. No data sent or received */
#define SAL_READ_CONST_DATA_COMMAND  0x02 /* return smxConstDataTable, set Length to the size of smxConstDataTable. Target clears reset bit */
#define SAL_READ_DATA_COMMAND        0x03 /* return data at Address size of Length */
#define SAL_WRITE_DATA_COMMAND       0x04 /* set target data at Address size of Length */
#define SAL_CAPTURE_COMMAND          0x05 /* stop GAT event buffer */
#define SAL_CAPTURE_DONE_COMMAND     0x06 /* start GAT event buffer again*/
#define SAL_ECHO_COMMAND             0x07 /* echo Address, Length, and Data back to client */

/* status from the target */
#define SAL_BIG_ENDIAN_TARGET        0x00010000 /* Endian of target */
#define SAL_CAPTURE_REQUEST_TARGET   0x00020000 /* target has requested a capture, cleared by SAL_CAPTURE_DONE_COMMAND */
#define SAL_UNKNOWN_COMMAND_TARGET   0x00040000 /* target did not recognize the command from the client */
#define SAL_RECENT_RESET_OF_TARGET   0x00080000 /* target has been reset since to last SAL_READ_CONST_DATA_COMMAND command */

#define SAL_COMMAND_MASK             0x0000FFFF
#define SAL_STATUS_MASK              0xFFFF0000

/*****************************************************************************
* Commands are sent from the smxAware Live client to the smx target (smxAware Live server).
* All will also include Address followed by Length followed by Data (if any).
* The smx target parse the command, add it's status bits to the Command and
* return Command Address Length and Data.
*
* The command will be converted by htonl so it will always be in network byte order.
* That way the Endian bit can always be read by the client.
* The client will convert the Address, Length, and Data to match the target.
*
*****************************************************************************/
typedef struct
{
  #if !defined(SMXAWARE_LIVE_CLIENT)    /* only defined in the smxAware Live client */
   u32  messageLength; /* length of Command + Address + Length + Data, excludes messageLength (network byte order) */
  #endif
   u32  command;       /* (network byte order) */
   u32  address;       /* address of requested data, 0 if unused by command (target Endian byte order) */
   u32  length;        /* length  of requested data, 0 if unused by command (target Endian byte order) */
   u32  data;          /* requested data (target Endian byte order) */
} smxCommandFormatStruct;


typedef struct
{
   char symName[SMXAWARE_SYM_NAME_LEN];
   u32  symAddress;
   u32  dataValue;
   u32  isStatic;  /* TRUE if the variable is static and u32. The smxAware Live client reads and caches static variable  */
                   /* from target once at startup using the SAL_READ_CONST_DATA_COMMAND command. */
                   /* If the variable is not a u32, then isStatic must be set to FALSE so that the smxAwareLiveTaskMain() won't */
                   /* do a u32 read of a u8 variable causing a runtime data fault */
} smxLiveSymTableStruct;


/* constant data, to be read by the smxAware Live client first */
typedef struct
{
   u32  smx_Version;
   u32  smx_Processor;
   u32  smx_NewScheduler;
   u32  smxAwareDataVersion;
   u32  smxSymTableLength;
   u32  smxSymTableEntries;
   u32  smxSymTableAddress;
   u32  smxLiveHandleTableAddress;
   u32  smxLiveHandleTableBytes;
   u32  sa_booleansz;

   // if you need to add a new variable that is read from the target, rename one of these place holders
   // so the length of this data structure will remain constant.
   u32  placeHolder1;
   u32  placeHolder2;
   u32  placeHolder3;
   u32  placeHolder4;
   u32  placeHolder5;
   u32  placeHolder6;
   u32  placeHolder7;
   u32  placeHolder8;
   u32  placeHolder9;
} smxLiveConstDataTableStruct;



#ifdef __cplusplus
extern "C" {
#endif
/* These variables must be public so smxAware Live can read them. */
u32 sa_systemStackTop;
u32 sa_systemStackBottom;
u32 sal_etime;
u8 salHandleName[HT_SIZE+1][SAL_NAME_LEN+2]; /* add 2 to SAL_NAME_LEN so each row will be 24 bytes for easy transfer to client */

extern SA_PRINT sa_print;
extern u32      sa_ready;
extern u32      sb_stk_fill_val;
extern TCB_PTR* sa_smx_ct_address;  /* Points to global smx_ct. Needed in case app define local var smx_ct. */
extern u32      smx_rtcb_size;
extern u32      cf_max_priority;
extern u32      sa_SMX_NUM_ERRORS;
#ifdef __cplusplus
}
#endif


/* symbol table, to be read by the smxAware Live client after smxConstDataTable */
smxLiveSymTableStruct smxLiveSymTable[] =
{
   {"sa_print",            (u32) &sa_print,              0x00, TRUE},
   {"sa_SMX_NUM_ERRORS",   (u32) &sa_SMX_NUM_ERRORS,     0x00, TRUE},
   {"sa_systemStackTop",   (u32) &sa_systemStackTop,     0x00, TRUE},
   {"sa_systemStackBotto", (u32) &sa_systemStackBottom,  0x00, TRUE},
   {"smx_cf",              (u32) &smx_cf,                0x00, TRUE},
   {"cf_max_priority",     (u32) &cf_max_priority,       0x00, TRUE},
   {"smx_ct",              (u32) &smx_ct,                0x00, TRUE},
   {"smx_etime",           (u32) &smx_etime,             0x00, FALSE},
   {"sal_etime",           (u32) &sal_etime,             0x00, FALSE},
   {"smx_errctr",          (u32) &smx_errctr,            0x00, TRUE},
  #if SMX_CFG_ERROR_MSGS
   {"smx_errmsg",          (u32) &smx_errmsg,            0x00, TRUE},   /* smx_errmsg is a pointer to an array of error messages */
  #endif
   {"smx_errno",           (u32) &smx_errno,             0x00, FALSE},
   {"smx_ebi",             (u32) &smx_ebi,               0x00, TRUE},
   {"smx_ebx",             (u32) &smx_ebx,               0x00, TRUE},
   {"smx_evben",           (u32) &smx_evben,             0x00, TRUE},
   {"smx_evbi",            (u32) &smx_evbi,              0x00, TRUE},
   {"smx_evbn",            (u32) &smx_evbn,              0x00, FALSE},
   {"smx_evbx",            (u32) &smx_evbx,              0x00, TRUE},
   {"sb_ticktmr_cntpt",    (u32) &sb_ticktmr_cntpt,      0x00, TRUE},
   {"sb_ticktmr_clkhz",    (u32) &sb_ticktmr_clkhz,      0x00, TRUE},
   {"smx_htchg",           (u32) &smx_htchg,             0x00, FALSE},
   {"smx_htpres",          (u32) &smx_htpres,            0x00, FALSE},  /* smx_htpres is set to FALSE because it could be a byte variable */
   {"smx_hti",             (u32) &smx_hti,               0x00, TRUE},
   {"smx_htn",             (u32) &smx_htn,               0x00, TRUE},
   {"smx_rq",              (u32) &smx_rq,                0x00, TRUE},
   {"smx_rqtop",           (u32) &smx_rqtop,             0x00, TRUE},
  #if SMX_CFG_PROFILE
   {"smx_pidle",           (u32) &smx_pidle,             0x00, FALSE},
   {"smx_pwork",           (u32) &smx_pwork,             0x00, FALSE},
   {"smx_povhd",           (u32) &smx_povhd,             0x00, FALSE},
  #endif
   {"sa_ready",            (u32) &sa_ready,              0x00, FALSE},
   {"smx_spmin",           (u32) &smx_spmin,             0x00, TRUE},
   {"smx_heapmin",         (u32) &smx_heapmin,           0x00, TRUE},
   {"smx_heapmax",         (u32) &smx_heapmax,           0x00, TRUE},
   {"sb_stk_fill_val",     (u32) &sb_stk_fill_val,       0x00, TRUE},
   {"sa_smx_ct_address",   (u32) &sa_smx_ct_address,     0x00, TRUE},
   {"smx_rtcb",            (u32) &smx_rtcb,              0x00, FALSE},
   {"smx_rtcb_size",       (u32) &smx_rtcb_size,         0x00, TRUE},
   {"smx_heaphwm",         (u32) &smx_heaphwm,           0x00, FALSE},
   {"smx_freestack",       (u32) &smx_freestack,         0x00, FALSE},
   {"smx_lqctr",           (u32) &smx_lqctr,             0x00, FALSE},
   {"smx_lqhwm",           (u32) &smx_lqhwm,             0x00, FALSE},
   {"smx_timeout",         (u32) &smx_timeout,           0x00, TRUE},  /* smx_timeout is an array of timers */
   {"smx_timer",           (u32) &smx_timeout,           0x00, TRUE},
   
   /* added for v4.2 and later */
   {"smx_rqx",             (u32) &smx_rqx,               0x00, TRUE},
   {"smx_bcbs",            (u32) &smx_bcbs,              0x00, TRUE},
   {"smx_mcbs",            (u32) &smx_mcbs,              0x00, TRUE},
   {"smx_mucbs",           (u32) &smx_mucbs,             0x00, TRUE},
   {"smx_pcbs",            (u32) &smx_pcbs,              0x00, TRUE},
   {"smx_picbs",           (u32) &smx_picbs,             0x00, TRUE},
   {"smx_qcbs",            (u32) &smx_qcbs,              0x00, TRUE},
   {"smx_tcbs",            (u32) &smx_tcbs,              0x00, TRUE},
   {"smx_tmcbs",           (u32) &smx_tmcbs,             0x00, TRUE},

  #ifdef ENABLE_SMXOBJECT_SYMBOLS
   {"smx_tq",              (u32) &smx_tq},     /* timer queue */
   {"smx_lqi",             (u32) &smx_lqi},
   {"smx_lqx",             (u32) &smx_lqx},
   {"smx_lqin",            (u32) &smx_lqin},
   {"smx_lqout",           (u32) &smx_lqout},
   {"smx_bcbi",            (u32) &smx_bcbs.pi},
   {"smx_bcbn",            (u32) &smx_bcbs.pn},
   {"smx_bcbx",            (u32) &smx_bcbs.px},
   {"smx_mcbi",            (u32) &smx_mcbs.pi},
   {"smx_mcbn",            (u32) &smx_mcbs.pn},
   {"smx_mcbx",            (u32) &smx_mcbs.px},
   {"smx_mucbi",           (u32) &smx_mucbs.pi},
   {"smx_mucbn",           (u32) &smx_mucbs.pn},
   {"smx_mucbx",           (u32) &smx_mucbs.px},
   {"smx_pcbi",            (u32) &smx_pcbs.pi},
   {"smx_pcbn",            (u32) &smx_pcbs.pn},
   {"smx_pcbx",            (u32) &smx_pcbs.px},
   {"smx_picbi",           (u32) &smx_picbs.pi},
   {"smx_picbn",           (u32) &smx_picbs.pn},
   {"smx_picbx",           (u32) &smx_picbs.px},
   {"smx_qcbi",            (u32) &smx_qcbs.pi},
   {"smx_qcbn",            (u32) &smx_qcbs.pn},
   {"smx_qcbx",            (u32) &smx_qcbs.px},
   {"smx_tcbi",            (u32) &smx_tcbs.pi},
   {"smx_tcbn",            (u32) &smx_tcbs.pn},
   {"smx_tcbx",            (u32) &smx_tcbs.px},
   {"smx_tmcbi",           (u32) &smx_tmcbs.pi},
   {"smx_tmcbn",           (u32) &smx_tmcbs.pn},
   {"smx_tmcbx",           (u32) &smx_tmcbs.px},
  #endif
};
#define SMX_SYMBOL_TABLE_ENTRIES   (sizeof(smxLiveSymTable)/sizeof(smxLiveSymTableStruct))
#define SMX_SYMBOL_TABLE_SIZE      (sizeof(smxLiveSymTable))

smxLiveConstDataTableStruct smxLiveConstDataTable =
{
   SMX_VERSION,      /* 0x400 or greater */
   SB_PROCESSOR,
  #ifdef SMX_CFG_SCHEDN
   TRUE,
  #else
   FALSE,
  #endif
   SMXAWARE_DATA_VERSION,
   SMX_SYMBOL_TABLE_SIZE,
   SMX_SYMBOL_TABLE_ENTRIES,
   (u32) &smxLiveSymTable,
   (u32) &salHandleName,
   (u32) sizeof(salHandleName),
   sizeof(BOOLEAN)
};

#endif /* SMXAWARE_LIVE */

/*
   Notes:

   1. Each ring buffer entry starts with a u32 TimeOfDispatch. Following
      TimeOfDispatch is a null terminated string. TimeOfDispatch may run
      over the end of the buffer by up to 4 bytes so 4 bytes is added to
      the buffer size.
*/

#endif /* SMXAWARE */
#endif /* SMX_SMXAWARE_H */


