/*
* uartif7.c                                                 Version 5.2.0
*
* UART Interrupt-Driven Low-Level Driver Functions for STM32 Processor with
* HAL library.
*
* Copyright (c) 2012-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: XiaoYi Chen, Xingsheng Wan
*
*****************************************************************************/

#include "smxbase.h"

#if SB_CFG_UARTI

#include "buart.h"
#include "buarthw.h"
#include "bsp.h"

#define RX_EVT      0x1
#define TX_EVT      0x2
#define UART_NUM    7

static UART_HandleTypeDef *UartHandle[UART_NUM];
static LCB_PTR UARTInterruptLSR;

SUR_HANDLE_T *UART_SURH[UART_NUM] = {0, 0, 0, 0, 0, 0, 0};
#if !defined(USART6)
#define USART6 0
#define USART6_IRQn 0
#endif
#if defined(SMX_TXPORT) && defined(SMX_STM32CUBEMX)
extern UART_HandleTypeDef huart1;
#else
static USART_TypeDef *UartPort[UART_NUM] = {0, USART1, USART2, USART3, 0, 0, USART6};
#endif
static int UartIRQn[UART_NUM] = {0, USART1_IRQn, USART2_IRQn, USART3_IRQn, 0, 0, USART6_IRQn};

static void UARTInterruptLSRMain(uint iParameter)
{
    SUR_HANDLE_T *pHandle = (SUR_HANDLE_T *)iParameter;
    if(pHandle->flag & RX_EVT)
    {
        pHandle->flag &= ~RX_EVT;
        if(pHandle->pInCallback)
        {
            uint expected = pHandle->InExpected;
            pHandle->InExpected = 0;
            pHandle->pInCallback(expected);
            pHandle->pInCallback = NULL;
        }
        else
        {
            smx_SemSignal(pHandle->pInSem);
        }
    }
    if(pHandle->flag & TX_EVT)
    {
        pHandle->flag &= ~TX_EVT;
        if(pHandle->pOutCallback)
        {
            uint expected = pHandle->OutExpected;
            pHandle->OutExpected = 0;
            pHandle->pOutCallback(expected);
            pHandle->pOutCallback = NULL;
        }
        else
        {
            smx_SemSignal(pHandle->pOutSem);
        }
    }
}

int UARTInterruptHandler(uint iParameter)
{
    u8 data8;
    SUR_HANDLE_T *pHandle = (SUR_HANDLE_T *)iParameter;
    UART_HandleTypeDef *pUart = UartHandle[pHandle->port];

    /* check and clear RX overrun */
    if(__HAL_UART_GET_IT(pUart, UART_IT_ORE))
    {
        __HAL_UART_CLEAR_OREFLAG(pUart);
    }

    if(__HAL_UART_GET_IT(pUart, UART_IT_RXNE) && __HAL_UART_GET_IT_SOURCE(pUart, UART_IT_RXNE))
    {
        data8 = (u8)(pUart->Instance->RDR & 0xFF);
        if(sur_InFreeSize(pHandle))
        {
            sur_InWriteByte(pHandle, data8);
            if(pHandle->InTerm != (uint)-1)
            {
                if((uint)data8 == pHandle->InTerm)
                {
                    pHandle->InTerm = (uint)-1;
                    pHandle->InExpected = sur_InAvail(pHandle);
                }
            }
            if(pHandle->InPending && sur_InAvail(pHandle) >= pHandle->InExpected)
            {
                pHandle->flag |= RX_EVT;
                pHandle->InPending = FALSE;
            }
        }
        else
        {
            /* buffer overflow */
            sur_InWriteByte(pHandle, data8);
            /* TODO flow control or disable RX here */
        }
    }

    if(__HAL_UART_GET_IT(pUart, UART_IT_TXE) && __HAL_UART_GET_IT_SOURCE(pUart, UART_IT_TXE))
    {
        /* loop sending data */
        while (sur_OutAvail(pHandle))
        {
            if (__HAL_UART_GET_FLAG(pUart, UART_FLAG_TXE))
            {
                sur_OutReadByte(pHandle, &data8);
                pUart->Instance->TDR = data8;
            }
            else
            {
                break;
            }
        }

        /* handle callback when no more data to send */
        if (!sur_OutAvail(pHandle))
        {        
            if(pHandle->OutPending)
            {
                pHandle->flag |= TX_EVT;
                pHandle->OutPending = FALSE;
            }
            
            /* disable TXRDY interrupt when no more data to send */
            __HAL_UART_DISABLE_IT(pUart, UART_IT_TXE); 
        }
    }

    if(pHandle->flag != 0)
    {
        smx_LSR_INVOKE(UARTInterruptLSR, iParameter);
    }

    return 1;
}

void sur_HWInit(uint port, u32 baudrate, uint parity, uint dbit, uint sbit, SUR_HANDLE_T *phdl)
{
    UART_SURH[port] = phdl;

#if defined(SMX_TXPORT) && defined(SMX_STM32CUBEMX)
    UartHandle[port] = &huart1;
#else
    UartHandle[port] = (UART_HandleTypeDef*)smx_HeapMalloc(sizeof(UART_HandleTypeDef));
#endif
    phdl->IRQ = UartIRQn[port];

#if !(defined(SMX_TXPORT) && defined(SMX_STM32CUBEMX)) /* UART init done in CubeMX generated code MX_USART1_UART_Init() */
    UartHandle[port]->Instance = UartPort[port];
    UartHandle[port]->Init.BaudRate = baudrate;
    UartHandle[port]->Init.HwFlowCtl = UART_HWCONTROL_NONE;
    UartHandle[port]->Init.Mode = UART_MODE_TX_RX;
    UartHandle[port]->Init.OverSampling = UART_OVERSAMPLING_16;
    UartHandle[port]->Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
//#if 1 /* Set to 0 if HAL USART_InitTypeDef doesn't have ClockPrescaler or Prescaler field. */
#if defined(SB_CPU_STM32H7XX) || defined(SB_CPU_STM32U5XX)
   #if 1
    UartHandle[port]->Init.ClockPrescaler = 0;
   #else /* old HAL */
    UartHandle[port]->Init.Prescaler = 0;
    UartHandle[port]->Init.FIFOMode = UART_FIFOMODE_ENABLE;
    UartHandle[port]->Init.RXFIFOThreshold = UART_RXFIFO_THRESHOLD_1_4;
   #endif
#endif
#if defined(USE_HAL_DRIVER_V1_4)
    UartHandle[port]->gState = HAL_UART_STATE_RESET;
#else
    UartHandle[port]->State = HAL_UART_STATE_RESET;
#endif
    UartHandle[port]->AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;

    switch(parity)
    {
    case SUR_PARITY_ODD:
        UartHandle[port]->Init.Parity = UART_PARITY_ODD;
        break;
    case SUR_PARITY_EVEN:
        UartHandle[port]->Init.Parity = UART_PARITY_EVEN;
        break;
    case SUR_PARITY_NONE:
    default:
        UartHandle[port]->Init.Parity = UART_PARITY_NONE;
        break;
    }
    switch(sbit)
    {
    case SUR_STOP_BITS_1:
        UartHandle[port]->Init.StopBits = UART_STOPBITS_1;
        break;
    case SUR_STOP_BITS_1_5:
        UartHandle[port]->Init.StopBits = UART_STOPBITS_1_5;
        break;
    case SUR_STOP_BITS_2:
        UartHandle[port]->Init.StopBits = UART_STOPBITS_2;
        break;
    }
    switch(dbit)
    {
    case 9:
        UartHandle[port]->Init.WordLength = UART_WORDLENGTH_9B;
        break;
    case 8:
    default:
        UartHandle[port]->Init.WordLength = UART_WORDLENGTH_8B;
        break;
    }

    /* currently the driver only supports the following settings */
    if(!((UartHandle[port]->Init.Parity == UART_PARITY_NONE) &&
         (UartHandle[port]->Init.WordLength == UART_WORDLENGTH_8B) &&
         (UartHandle[port]->Init.StopBits == UART_STOPBITS_1)))
    {
        /* error */
        return;
    }

    /* init and enable USARTs */
    if(HAL_UART_Init(UartHandle[port]) != HAL_OK)
    {
        /* error */
        return;
    }

    /* hook interrupt and create LSR */
    sb_ISRInstall(phdl->IRQ, (u32)phdl, UARTInterruptHandler, "UART ISR");
    if (UARTInterruptLSR == NULL)
        UARTInterruptLSR = smx_LSRCreate(UARTInterruptLSRMain, NULL, 0, SMX_FL_TRUST, "UARTInterruptLSR", &UARTInterruptLSR);

    /* enable receive interrupt only */
    __HAL_UART_ENABLE_IT(UartHandle[port], UART_IT_RXNE);
 #if defined (SB_CPU_STM32H7XX)
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_WUF);
 #endif
 #if defined(SB_CPU_STM32H7XX) || defined(SB_CPU_STM32U5XX)
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_IDLE);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_LBD);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_CTS);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_CM);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_WUF);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_TXFE);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_RXFT);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_TXFT);
 #endif
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_TXE);
    __HAL_UART_DISABLE_IT(UartHandle[port], UART_IT_TC);

    /* finally enable the interrupt */
    sb_IRQUnmask(phdl->IRQ);
#endif /* SMX_TXPORT  && SMX_STM32CUBEMX */
}

void sur_HWRelease(uint port)
{
    if(HAL_UART_DeInit(UartHandle[port]) != HAL_OK)
    {
        /* error */
    }
    smx_HeapFree(UartHandle[port]);
}

void sur_HWStartXmit(uint port, SUR_HANDLE_T *phdl)
{

     /* send first byte */
     if (__HAL_UART_GET_FLAG(UartHandle[port], UART_FLAG_TC) && sur_OutAvail(phdl))
     {
         u8 data8;
         sur_OutReadByte(phdl, &data8);
         UartHandle[port]->Instance->TDR = data8;
     }

    /* enable transmit interrupt */
    __HAL_UART_ENABLE_IT(UartHandle[port], UART_IT_TXE);
}

#endif /* SB_CFG_UARTI */

