/*
* smxmain.c                                                 Version 5.2.0
*
* !!! THIS MODULE IS NOT OPTIONAL !!!
*
* PROTOSYSTEM MAIN MODULE. Includes smx configuration table, initialization
* code, smx_Idle, and exit code. All routines must be present, but their
* content can be changed as necessary.
*
* "USER" marks places in this module and others that you may wish to
* change for your system.
*
* Copyright (c) 1989-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore and Ralph Moore
*
*****************************************************************************/

#include "smx.h"
#include "smxmain.h"

/* event buffer */
#pragma section = "EVB"
#define EVB_SIZE  ((u32)__section_size("EVB"))  /* size of event buffer in bytes */

/* main heap bin definitions */
u32 const heap0_binsz[] =
/* small bins  0    1    2    3    4    5    6    7    8    9   10   11   12 */ \
             {24,  32,  40,  48,  56,  64,  72,  80,  88,  96, 104, 112, 120, \
/* large bins 13   14   15   16   17   18   19   20   21   22   23   24   25   26   27   28 */ \
             128, 256, 384, 512, 640, 768, 896,1024,1152,1280,1408,1536,1664,1792,1920,2048, -1};

/* main heap variables */
HBCB        heap0_bin[(sizeof(heap0_binsz)/4)-1];  /* heap0 bins */
u32         heap0_hn;                              /* heap0 number */
EHV         heap0_hv;                              /* heap0 variable struct */

#if EH_STATS
u32         heap0_bnum[(sizeof(heap0_binsz)/4)-1]; /* heap0 number of chunks per bin */
u32         heap0_bsum[(sizeof(heap0_binsz)/4)-1]; /* heap0 sum of chunk sizes per bin */
#endif

/* main heap allocation */
#if SMX_CFG_MPU
#pragma default_variable_attributes = @ ".heap0"
#endif
#pragma data_alignment = 16
u8          heap0[SMX_CFG_HEAP_SPACE];

/* fs partition heap definitions */
#if SMX_CFG_MPU
#pragma default_variable_attributes = @ ".fs_rodata"
u32 const   fs_binsz[] = /* single bin heap */
/* bin  0       end */
      {24, -1};
#pragma section = "fs_heap"
#pragma default_variable_attributes = @ ".fs_bss"
HBCB        fs_bin[(sizeof(fs_binsz)/4)-1];  /* fs_heap bins */
#if EH_STATS
u32         fs_bnum[(sizeof(fs_binsz)/4)-1]; /* fs_heap number of chunks per bin */
u32         fs_bsum[(sizeof(fs_binsz)/4)-1]; /* fs_heap sum of chunk sizes per bin */
#endif
u32         fs_hn;                           /* fs_heap number */
EHV         fs_hv;                           /* fs_heap variable struct */
#endif /* SMX_CFG_MPU */

#if SMX_CFG_MPU
#pragma default_variable_attributes = @ ".sys_bss"
#endif

#if SMX_CFG_PROFILE
u32         smx_rtcb[SMX_CFG_RTCB_SIZE][SMX_CFG_NUM_TASKS + 5];
#endif

void        smx_HeapManager(void);
BOOLEAN     smx_StartupChecks(void);


/******* MAIN
*
* Entry point from assembly startup code.
*
*****************************************************************************/

#if defined(SMX_STM32CUBEMX)
void smx_main(void)
#else
int main(void)
#endif
{
   sb_INT_DISABLE();

   if (!smx_StartupChecks())
      sb_Exit(SMXE_ABORT);

  #if SMX_CFG_MPU && defined(SMX_BT_DEBUG)
   smx_mmf_halt = TRUE;       /* halt on MMF if debugging */
  #endif

   if (eh_hvpn == 0)
      smx_HeapsInit();        /* init heaps if not already done by initializer */

  #if defined(SB_HW_INIT_AT_MAIN) && !(defined(SMX_TXPORT) && defined(SMX_STM32CUBEMX))
   sb_HWInitAtMain();         /* HW init that must be done after compiler startup code <6> */
  #endif

  #if !(defined(SMX_TXPORT) && defined(SMX_STM32CUBEMX)) /* already done in startup code for TXPort */
   sb_TickInit();             /* init tick used for all smx timing and ptime <2> <3> */
  #endif

   sb_IRQsMask();             /* mask all interrupts and save current mask, until ainit() */

  #if SMX_CFG_MPU
   mp_MPUInit();              /* initialize the MPU */
   sb_INT_ENABLE();           /* needed for SVC in sb_TM_INIT() else hard fault; ok since IRQs masked above */
  #endif

   sb_TM_INIT();              /* init precise time measurement routines */
   sb_StimeSet();             /* Initialize smx_stime. Must be done after C/C++ initializers run. */
   smx_Go();                  /* initialize smx and enter scheduler */

/* execution should not go below here */

  #if SMX_CFG_MPU
   heap0[0] = 1;              /* prevent being dead-stripped */
  #endif

   return(0);                 /* keep compiler happy */
}


/******* APPLICATION INITIALIZATION
* This is the starting point for the task environment. smx_Go() starts
* smx_Idle using ainit() as its code and gives it maximum priority so it
* cannot be preempted until done. At the end of ainit(), the smx_Idle main 
* function is changed to smx_IdleMain(), and its priority is lowered to PRI_MIN.
*
* DO NOT MAKE SMX CALLS THAT WAIT -- ainit() must be allowed to finish
* before other tasks run.
*****************************************************************************/

void ainit(u32)
{
   sb_SSFill();               /* fill whole SS with pattern */

   sb_IRQsUnmask();           /* restore mask from start of main() */
   sb_TickIntEnable();        /* enable tick interrupt */

  #if CP_PORTAL
   cp_init(CPS_SLOTS);
  #endif

  #if SMX_CFG_PROFILE
   smx_rtcbi = &smx_rtcb[0][0];
   smx_ProfileInit();         /* initialize rtc buffer and pointers */
  #endif

  #if SMX_CFG_MPU
   sb_INT_ENABLE();           /* needed for SVC in periphal init else hard fault */
  #endif

   sb_PeripheralsInit();      /* initialize peripherals */
   sb_ConsoleOutInit();       /* initialize UART after sb_PeripheralsInit() */

   /* Exit if any errors occurred during smx init in smx_Go() or scheduler.
      Call after sb_ConsoleOutInit() so aexit() can display errors. */
   if (smx_errno)
      aexit(smx_errno);

   sb_INT_ENABLE();           /* test point; see note 1 at end of file */
   sb_ConsoleInInit();        /* initialize console input device */
   appl_init();               /* main application initialization routine */

  #if SMX_CFG_EVB
   smx_evbn = smx_evbi;       /* start event monitoring */
  #endif

   smx_Idle->flags.strt_lockd = 0;  /* clear the start locked flag for idle */
   smx_TaskStartNew(smx_ct, 0, PRI_MIN, smx_IdleMain);  /* become normal idle task */

/* code below here should not execute */

   sb_MsgOutConst(SB_MSG_ERR, "Error in ainit(): smx_TaskStartNew() returned");
   sb_DEBUGTRAP();
   sb_Exit(SMXE_ABORT);
}

#if SMX_CFG_MPU
/*............................. SECTION CHANGE .............................*/
#pragma default_function_attributes = 
#endif

/******* smx_IdleMain
* smx_Idle runs when there is no other task to run. It can be used to
* compute processor utilization as well as to gather other information
* of interest. It can also be used to perform confidence tests such as
* heap checks, stack checks, etc.
*
* Do not add code within the SMX_CFG_STACK_SCAN conditional.
*****************************************************************************/

void smx_IdleMain(u32)
{
   #define HEAP_MGR_CNT 3

   static u32 heap_mgr_cnt;
   static u32 hn;

   #if CP_PORTAL
   /* open console portal for idle */
   mp_FPortalOpen(&cpcli_idle, CP_SLOT_IDLE, 80, 1, PRI_LO, 5, "cp_rxchg"); /*<7>*/
   #endif

   while(TRUE)
   {
     #if SMX_CFG_STACK_SCAN
      smx_StackScan();

      #if defined(SMX_BT_DEBUG)
      /* check if system stack needs to be larger */
      if (sb_SS_GET_SIZE() - smx_sshwm < 40)
         sb_DEBUGTRAP();
      #endif

      if (smx_idleup)
      {
         smx_idleup = FALSE;
         smx_NullSSR(); /* return to scheduler */
      }
     #endif

      smx_EtimeRollover();
      sb_MsgDisplay();

     #if SMX_CFG_PROFILE
      smx_ProfileDisplay();
     #endif

      /* call heap manager for next heap */
      if (smx_hmng)  /* set periodically by smx_KeepTimeLSR() */
      {
         if (++heap_mgr_cnt == HEAP_MGR_CNT)
         {
            if (eh_hvp[hn]->mgr)
               eh_hvp[hn]->mgr();
            if (++hn >= eh_hvpn)
               hn = 0;
            heap_mgr_cnt = 0;
         }
         smx_hmng = FALSE;
      }

     #if SMX_CFG_RTLIM
      /* test for end of RTLIM frame <9> */
      if (++smx_Idle->idle_ctr >= SMX_CFG_IDLE_RTLIM)
      {
         if (smx_rtlimsem->fl != NULL)
         {
            smx_LSRsOff();
            /* resume all tasks at rtlimsem */
            smx_SemClear(smx_rtlimsem);

            /* clear all non-child RTLIM counters <6> */
            TCB* task = (TCB_PTR)smx_tcbs.pi;
            u32* limp = &task->rtlim; 
            u32* parp = (u32*)&task->parent;

            for (u32 n = 0; n < SMX_CFG_NUM_TASKS; n++)
            {
               if (*limp > 0 && *parp == 0)
                  *(limp + 1) = 0;
               limp += sizeof(TCB)/4;
               parp += sizeof(TCB)/4;
            }
            smx_LSRsOn();
         }
         smx_Idle->idle_ctr = 0;
      }
     #endif

      /* bump to allow another waiting PRI_MIN task to run */
      if (smx_ct->fl != (CB_PTR)&smx_rq[PRI_MIN])
         smx_TaskBump(smx_ct, SMX_PRI_NOCHG);
      else
      {
        #if SMX_CFG_STACK_SCAN
         /* do not power down if stacks are waiting to be scanned */
         if (smx_scanstack == NULL)
        #endif
            smx_SysPowerDown(0); /* <5> */
      }
   }
}

#if SMX_CFG_MPU
/*............................. SECTION CHANGE .............................*/
#pragma default_function_attributes = @ ".text"  /* <8> */
#endif

/******* smx_HeapsInit
* Initialize main heap (0) and other heaps required for C++ initializers.
* Other heaps can be initialized here, if desired.
*****************************************************************************/

BOOLEAN smx_HeapsInit(void)
{
   /* set heap timeout (applies to all heaps) */
   smx_htmo = SMX_TMO_INF;

   /* create main heap and initialize it */
   memset((void*)&heap0_hv, 0, sizeof(EHV));
   heap0_hv.name = "main (heap0)";
   heap0_hv.bszap = (u32*)heap0_binsz;
   heap0_hv.binp  = (HBCB*)heap0_bin;
   heap0_hn = smx_HeapInit(SMX_CFG_HEAP_SPACE, SMX_CFG_HEAP_DC_SIZE, (u8*)SMX_CFG_HEAP_ADDRESS, &heap0_hv, EH_NORM);
   if (heap0_hn == -1)
   {
      smx_ERROR_SS(SMXE_HEAP_INIT_FAIL, SMX_ERRH_SMX)
      sb_DEBUGTRAP();
      return FALSE;
   }
   heap0_hv.mgr = smx_HeapManager;
  #if SMX_BT_DEBUG
   heap0_hv.mode.fl.fill = ON;
  #endif
  #if SB_MIN_RAM
   heap0_hv.mode.fl.amerge = OFF;
   heap0_hv.mode.fl.cmerge = ON;
  #endif
  #if EH_STATS
   heap0_hv.bnump = (u32*)&heap0_bnum;
   heap0_hv.bsump = (u32*)&heap0_bsum;
  #endif

  #if SMX_CFG_MPU
   /* create fs heap and initialize it */
   #pragma section = "fs_heap"
   memset((void*)&fs_hv, 0, sizeof(EHV));
   fs_hv.name = "hp";
   fs_hv.bszap = (u32*)fs_binsz;
   fs_hv.binp  = (HBCB*)fs_bin;
   fs_hn = smx_HeapInit(__section_size("fs_heap"), 0, (u8*)__section_begin("fs_heap"), &fs_hv, EH_NORM);
   if (fs_hn == -1)
   {
      smx_ERROR_SS(SMXE_HEAP_INIT_FAIL, SMX_ERRH_SMX)
      sb_DEBUGTRAP();
      return FALSE;
   }
   fs_hv.mgr = NULL;
   #if SMX_BT_DEBUG
   fs_hv.mode.fl.fill = OFF;
   #endif
   #if EH_STATS
   fs_hv.bnump = (u32*)&fs_bnum;
   fs_hv.bsump = (u32*)&fs_bsum;
   #endif
  #endif /* SMX_CFG_MPU */
 
   return TRUE;
}

#if defined(__cplusplus)
#if defined(__IAR_SYSTEMS_ICC__)
/*
   Called from IAR startup code. Create heaps and smx block pools before C++
   global object constructors run. Must run after data init, so cannot be done
   from startup code. 
*/
extern "C" {
extern void $Super$$__call_ctors(void const *pibase, void const *ilimit);
void $Sub$$__call_ctors(void const *pibase, void const *ilimit)
{
   smx_CBPoolsCreate();
   if (eh_hvpn == 0)
      smx_HeapsInit();
   $Super$$__call_ctors(pibase, ilimit);
}
}
#endif /*__IAR_SYSTEMS_ICC__ */
#endif /* __cplusplus */

#if SMX_CFG_MPU
/*............................. SECTION CHANGE .............................*/
#pragma default_function_attributes = 
#endif

/******* smx_HeapManager
*
*  Heap management functions for main heap to run from smx_IdleMain(). This
*  code can be generalized for other heaps or reduced versions of it used.
*
*****************************************************************************/

void smx_HeapManager(void)
{
  #if SMX_CFG_MPU
   #define HEAP_SCAN_CNT 1
  #else
   #define HEAP_SCAN_CNT 5
  #endif

   static u32  i;
   static u32  heap_scan;
   CCB_PTR     tblcp;   /* top bin last chunk pointer */
   u32         tblcsz;  /* top bin last chunk size */
   u32         tcsz;    /* top chunk size */

   smx_HeapBinSort(100, 2, heap0_hn);

   /* Automatic heap chunk merge control */
   if(heap0_hv.mode.fl.amerge == ON)
   {
      tcsz = heap0_hv.tcp->sz;

      if ((heap0_hv.tbsz + tcsz) < (SMX_CFG_HEAP_SPACE/4))
         heap0_hv.mode.fl.cmerge = ON;
      else
      {
         tblcp = heap0_bin[heap0_hv.top_bin].fbl;
         tblcsz = (tblcp == NULL ? 0 : tblcp->sz);

         if (tblcsz < SMX_CFG_HEAP_CSZ_MAX && tcsz < SMX_CFG_HEAP_CSZ_MAX)
            heap0_hv.mode.fl.cmerge = ON;
         else
            heap0_hv.mode.fl.cmerge = OFF;
      }
   }
   /* Automatic heap scan and fix */
   if (++heap_scan == HEAP_SCAN_CNT)
   {
      smx_HeapScan(NULL, 4, 4, heap0_hn);
      heap_scan = 0;
   }
   /* Automatic heap bin scan and fix */
   if (smx_HeapBinScan(i, 2, 10, heap0_hn))
      i = (i == heap0_hv.top_bin ? 0 : i + 1);
}


#if SMX_CFG_MPU
/*............................. SECTION CHANGE .............................*/
#pragma default_function_attributes = @ ".text"  /* <8> */
#endif

/****** aexit
* Call application and smx exit routines. Note that errnum is the cause
* of exit, only. It indicates whether exit was due to a catastrophic
* error or a normal exit (user exit). It does not indicate whether other
* errors occurred while running, so it is possible to get a "normal exit"
* message, even if some (non-fatal) error occurred. Note that smx_errno
* reflects all error messages, catastrophic or otherwise.
*****************************************************************************/

void aexit(SMX_ERRNO errnum)
{
  #if !(SMX_CFG_ERROR_MSGS)
   #define BUF_SZ 24
   char buf[BUF_SZ];
  #endif

   smx_lockctr = 0; 

   if (smx_init)
   {
      appl_exit();  /* call main application exit routine */

     #if (SB_CON_OUT)
      /* Display any pending messages. */
      sb_MsgDisplay();
      /* Display cause of exit and first error in error buffer, if any.
         See note above about errnum. */
      sb_ConWriteString(0,EXIT_ROW,SB_CLR_LIGHTGRAY,SB_CLR_BLACK,!SB_CON_BLINK,"    Exit Due To: ");
      if (errnum == 0)
      {
         sb_ConWriteString(17,EXIT_ROW,SB_CLR_WHITE,SB_CLR_BLACK,!SB_CON_BLINK,"Normal Exit");
      }
      else
      {
        #if (SMX_CFG_ERROR_MSGS)
         sb_ConWriteString(17,EXIT_ROW,SB_CLR_WHITE,SB_CLR_BLACK,!SB_CON_BLINK,smx_errmsg[errnum]);
        #else
         strcpy(buf, "smx error (dec) #   ");
         _itoa(errnum, buf+17, 10);
         buf[BUF_SZ-1] = 0;  /* ensure terminated */
         sb_ConWriteString(17,EXIT_ROW,SB_CLR_WHITE,SB_CLR_BLACK,!SB_CON_BLINK,buf);
        #endif
      }
      if (sb_tick_init_done)
         sb_DelayMsec(100);  /* give time for interrupt-driven UART drivers to get message out */
      if (smx_errno)
      {
         sb_ConWriteString(0,EXIT_ROW+1,SB_CLR_LIGHTGRAY,SB_CLR_BLACK,!SB_CON_BLINK,"1st Error in EB: ");
        #if (SMX_CFG_ERROR_MSGS)
         if (smx_ebi->err < SMX_NUM_ERRORS)
            sb_ConWriteString(17,EXIT_ROW+1,SB_CLR_WHITE,SB_CLR_BLACK,!SB_CON_BLINK,smx_errmsg[smx_ebi->err]);
         #if SMX_CFG_PORTAL
         else
            sb_ConWriteString(17,EXIT_ROW+1,SB_CLR_WHITE,SB_CLR_BLACK,!SB_CON_BLINK,perrmsg[smx_ebi->err - SMX_NUM_ERRORS]);
         #endif
        #else
         strcpy(buf, "smx error (dec) #   ");
         _itoa(smx_ebi->err, buf+17, 10);
         buf[BUF_SZ-1] = 0;  /* ensure terminated */
         sb_ConWriteString(17,EXIT_ROW+1,SB_CLR_WHITE,SB_CLR_BLACK,!SB_CON_BLINK,buf);
        #endif

         if (sb_tick_init_done)
            sb_DelayMsec(5000);
      }
      #if CP_PORTAL
      cp_ConPortalClose(&cpcli_idle, 0);
      cp_exit(CPS_SLOTX);
      #endif
     #endif /* SB_CON_OUT */
   }
   sb_Exit(errnum);
}

/****** STARTUP CHECKS
* This routine does a few safety checks from the start of main().
* This is here to avoid cluttering main().
*****************************************************************************/

BOOLEAN smx_StartupChecks(void)
{
   /* Ensure SS has sufficient headroom and trap if SS too small. */
   if (sb_SSScan() < 40)
   {
      sb_DEBUGTRAP();
      return(FALSE);
   }

   /* Enum size checks. Must be done in code because preprocessor does not
      know about data types. If one of these triggers, see if compiler has
      switch to allow enums smaller than int, or set SB_CFG_SHORT_ENUMS to 0. */
   if ((sizeof(SMX_CBTYPE) != 1) || (sizeof(SMX_ERRNO) != 1) || (sizeof(SMX_STATE) != 1))
   {
      sb_DEBUGTRAP();
      return(FALSE);
   }

   /* Ensure BSS was cleared by the startup code by testing a few separate
      locations. Using bitwise | instead of || for efficiency. */
   if ((u32)smx_init | (u32)smx_srnest | (u32)smx_sched | (u32)smx_lqctr | (u32)smx_clsr)
   {
      sb_DEBUGTRAP();
      return(FALSE);
   }
   return(TRUE);
}

#if SMX_CFG_MPU
/*............................. SECTION CHANGE .............................*/
#pragma default_function_attributes = 
#endif

/****** TICK ISR
* Hooked to tick vector by sb_TickInit() called from main().
*****************************************************************************/

#if SMX_CFG_MPU
/* Note: Must do direct smx calls in ISRs not SVC calls (smx_ not smxu_) since
   SVC priority is lower (higher number) and would cause a hard fault. */
#endif

void smx_TickISR(void)
{
  #if SMX_ISR_C
   smx_ISR_ENTER();
  #endif
  #if SMX_CFG_PROFILE
   if (smx_rtc_frame_ctr == 1) /* if end of profile frame */
   {
      smx_i_rtc = smx_isr_rtc; /* capture isr rtc */
      smx_isr_rtc = 0;
      smx_l_rtc = smx_lsr_rtc; /* capture lsr rtc */
      smx_lsr_rtc = 0;
   }
  #endif
   smx_EVB_LOG_ISR(smx_TickISRH);
  #if !defined(SB_CPU_ARMM)
   sb_IRQClear(SB_TICK_IRQ);
  #endif
// sb_INT_ENABLE();  /* if this causes spurious interrupts, check sb_IRQClear() */

   smx_LSR_INVOKE(smx_KeepTimeLSR, 0);

   sb_IRQEnd(SB_TICK_IRQ);
   smx_TickISRHook();
   smx_EVB_LOG_ISR_RET(smx_TickISRH);
  #if SMX_ISR_C
   smx_ISR_EXIT();
  #endif
}

/****** TICK ISR HOOK
* Callback from the tick ISR for custom tick handling code. Useful to piggyback
* ISRs during development. Code used by esmx can be removed.
*****************************************************************************/

void smx_TickISRHook(void)
{
  #if (defined(SMX_ESMX) || defined(SMX_FRPORT))
   if (isren)
   {
      isren = FALSE;
      ISRFuncPtr();
   }
  #endif
  #if defined(SMX_TXPORT) && defined(SMX_STM32CUBEMX)
   HAL_IncTick();
  #endif
}

/****** ERROR MANAGER HOOK
* Callback from Error Manager for custom error handling code. Also can be used
* to set breakpoint to stop on error in order to look at parameters and call
* stack, to see where error occurred.
*
* Note: If using smx services it is necessary to protect against infinite
* looping in the event that a service experiences an error.
*****************************************************************************/

void smx_EMHook(SMX_ERRNO errnum, void *handle)
{
   //sb_DEBUGTRAP();           /* enable during debug, if desired */
   smx_Version = SMX_VERSION;  /* dummy statement so function not optimized out */
}

/****** ERROR MANAGER EXIT HOOK
* Callback from Error Manager due to an irrecoverable error. If handle is a
* task, stops it unless it has already been stopped.
*****************************************************************************/

void smx_EMExitHook(SMX_ERRNO errnum, void *handle)
{
   TCB_PTR t = NULL;
   smx_srnest = 0;  /* so will not return after TaskStop() */

   if (handle >= (void*)smx_tcbs.pi && handle <= (void*)smx_tcbs.px)
      t = (TCB_PTR)handle;

  #if SMX_CFG_MPU
   BOOLEAN stop_t = FALSE;
   if (errnum == SMXE_STK_OVFL)
   {
      if (t == NULL)
      {
         sb_DEBUGTRAP()  //USER: System Stack overflow. Reboot?
         while(1) {}
      }
      else
      {
         t->flags.stk_chk = 0; /* so sched will not test stack again */
         smx_TaskStop(t, SMX_TMO_INF);
         sb_MsgOutConst(SB_MSG_WARN, "TASK STOPPED");
      }
   }
   else if (errnum == SMXE_MMF_VIOL)
   {
      if (smx_clsr != 0)
      {
         sb_DEBUGTRAP()  //USER: MMF occurred in LSR. Decide what to do. Maybe skip to next LSR or reboot?
      }
      /* If not in SVC call, and in ISR/handler, don't stop task.
         (RETTOBASE == 0 means we were already in ISR so MMF nested. If in SVC
         call, we were in task since we already checked we're not in LSR. */
      else if (((*ARMM_NVIC_SYSHAN_CTRL_STATE & 0x80) == 0) && ((*ARMM_NVIC_INT_CTRL & ARMM_FL_RETTOBASE) == 0))
      {
         sb_DEBUGTRAP()  //USER: MMF occurred in ISR or handler. Decide what to do. Maybe reboot.
      }
      else if (smx_Idle != (TCB_PTR)smx_nullcb)
      {
         stop_t = TRUE;
      }
   }
   else
   {
      if (smx_ct == smx_Idle && errnum == SMXE_INIT_MOD_FAIL)
      {
         sb_DEBUGTRAP()  //USER: smx_modules_init() failed. Decide what to do. Maybe sb_Reboot().
         while(1) {}
      }
      else if (smx_ct != (TCB_PTR)&smx_dcb)
      {
         stop_t = TRUE;
      }
      else //USER: Fatal error occurred during startup. Decide what to do. Maybe sb_Reboot().
      {
         sb_DEBUGTRAP()  
         while(1) {}
      }
   }
   if (stop_t)
   {
      smx_TaskStop(t, SMX_TMO_INF);
      sb_MsgOutConst(SB_MSG_WARN, "TASK STOPPED");
   }
  #else
   if (errnum == SMXE_STK_OVFL)
   {
      if (t == NULL)
      {
         sb_DEBUGTRAP()  //USER: System Stack overflow. Reboot?
         while(1) {}
      }
      else if (smx_Idle != (TCB_PTR)smx_nullcb)
      {
         t->flags.stk_chk = 0; /* so sched will not test stack again */
         smx_TaskStop(t, SMX_TMO_INF);
         sb_MsgOutConst(SB_MSG_WARN, "TASK STOPPED");
      }
   }
  #endif /* SMX_CFG_MPU */
}

/*
   Notes:
   1. It is not necessary to explicitly call sb_INT_ENABLE() in ainit(),
      since interrupts will be enabled as soon as the first smx SSR is
      called. However, it is done so if the system hangs because interrupts
      are enabled, this will be clear. Otherwise, it might appear that the
      SSR caused the hang, which would result in wasted time debugging.
   2. The tick timer's counter is also used by sb_PtimeGet() for
      sb_DelayUsec(), smx_EVB (event buffer), smx_RTC (profiling), and
      sb_TM (time measurement) routines, so it is initialized early at the
      start of main(). If needed earlier, the call could be moved to the
      startup code, but mask the interrupt or don't enable until
      sb_TickIntEnable().
   3. sb_TickInit() unmasks the tick interrupt. It is purposely called
      before sb_MaskIRQs() which saves the current mask then masks all.
      The interrupt is unmasked (restored) by sb_UnmaskIRQs().
   4. IAR Go To Function doesn't work in some vers if { is in #if statement.
   5. Do not specify sleep mode unless processor actually stops.
   6. Chip vendor BSP library init routines may need to run at main(),
      after the compiler startup code has done data init from ROM for static
      initializers and possibly functions copied to RAM such as those that
      adjust flash config. bsp.h defines SB_HW_INIT_AT_MAIN when needed.
      For TXPort, it was already done in main().
   7. Client console portals can have only one pmsg in order for return values
     to be correct.
   8. init/exit code can be put in .text since it only runs once at start/end,
      so it is not needed during normal operation. This prevents it from
      being called again, and allows it to be used to fill gaps.
   9. An RTLIM frame ends after the Idle task has run SMX_CFG_IDLE_RTLIM times.
   10. Child task rtlim pointers are not cleared. Prior to clearing parent 
      rtl_ctrs, parent tasks that have hit rtlim could be logged.
*/
